@@ -1,5 +1,5 @@
 /* This file is part of KCachegrind.
-   Copyright (C) 2002, 2003 Josef Weidendorfer <Josef.Weidendorfer@gmx.de>
+   Copyright (C) 2002-2009 Josef Weidendorfer <Josef.Weidendorfer@gmx.de>
 
    KCachegrind is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -30,20 +30,14 @@
 #ifndef TREEMAP_H
 #define TREEMAP_H
 
-#include <qstring.h>
-#include <qwidget.h>
-#include <qpixmap.h>
-#include <q3ptrlist.h>
-#include <q3valuevector.h>
-#include <qcolor.h>
-#include <qapplication.h>
-#include <qstringlist.h>
-//Added by qt3to4:
+#include <QString>
+#include <QWidget>
+#include <QPixmap>
+#include <QColor>
+#include <QStringList>
 #include <QPaintEvent>
 #include <QKeyEvent>
-#include <QShowEvent>
 #include <QContextMenuEvent>
-#include <QResizeEvent>
 #include <kmenu.h>
 #include <QMouseEvent>
 #include <kconfiggroup.h>
@@ -51,9 +45,6 @@
 class TreeMapWidget;
 class TreeMapItem;
 class TreeMapItemList;
-class QString;
-
-class KConfigGroup;
 
 
 /**
@@ -95,6 +86,7 @@ public:
   virtual bool current() const { return false; }
   virtual bool shaded() const { return true; }
   virtual bool rotated() const { return false; }
+  virtual bool drawFrame() const { return true; }
 };
 
 
@@ -105,8 +97,8 @@ class StoredDrawParams: public DrawParams
 {
 public:
   StoredDrawParams();
-  StoredDrawParams(QColor c,
-                   bool selected = false, bool current = false);
+  explicit StoredDrawParams(const QColor& c,
+			    bool selected = false, bool current = false);
 
   // getters
   QString  text(int) const;
@@ -120,11 +112,12 @@ public:
   bool current() const { return _current; }
   bool shaded() const { return _shaded; }
   bool rotated() const { return _rotated; }
+  bool drawFrame() const { return _drawFrame; }
 
   const QFont& font() const;
 
   // attribute setters
-  void setField(int f, const QString& t, const QPixmap &pm = QPixmap(),
+  void setField(int f, const QString& t, const QPixmap& pm = QPixmap(),
                 Position p = Default, int maxLines = 0);
   void setText(int f, const QString&);
   void setPixmap(int f, const QPixmap&);
@@ -135,10 +128,15 @@ public:
   void setCurrent(bool b) { _current = b; }
   void setShaded(bool b) { _shaded = b; }
   void setRotated(bool b) { _rotated = b; }
+  void drawFrame(bool b) { _drawFrame = b; }
 
 protected:
   QColor _backColor;
-  bool _selected, _current, _shaded, _rotated;
+  bool _selected :1;
+  bool _current :1;
+  bool _shaded :1;
+  bool _rotated :1;
+  bool _drawFrame :1;
 
 private:
   // resize field array if needed to allow to access field <f>
@@ -151,7 +149,7 @@ private:
     int maxLines;
   };
 
-  Q3ValueVector<Field> _field;
+  QVector<Field> _field;
 };
 
 
@@ -167,7 +165,7 @@ private:
 class RectDrawing
 {
 public:
-  RectDrawing(QRect);
+  RectDrawing(const QRect&);
   ~RectDrawing();
 
   // The default DrawParams object used.
@@ -183,7 +181,7 @@ public:
   bool drawField(QPainter*, int f, DrawParams* dp = 0);
 
   // resets rectangle for free space
-  void setRect(QRect);
+  void setRect(const QRect&);
 
   // Returns the rectangle area still free of text/pixmaps after
   // a number of drawText() calls.
@@ -201,16 +199,12 @@ private:
 };
 
 
-class TreeMapItemList: public Q3PtrList<TreeMapItem>
+class TreeMapItemList: public QList<TreeMapItem*>
 {
 public:
   TreeMapItem* commonParent();
-protected:
-  int compareItems ( Item item1, Item item2 );
 };
 
-typedef Q3PtrListIterator<TreeMapItem> TreeMapItemListIterator;
-
 
 /**
  * Base class of items in TreeMap.
@@ -246,8 +240,8 @@ public:
 
   explicit TreeMapItem(TreeMapItem* parent = 0, double value = 1.0 );
   TreeMapItem(TreeMapItem* parent, double value,
-              const QString &text1, const QString &text2 = QString(),
-              const QString &text3 = QString(), const QString &text4 = QString());
+              const QString& text1, const QString& text2 = QString(),
+              const QString& text3 = QString(), const QString& text4 = QString());
   virtual ~TreeMapItem();
 
   bool isChildOf(TreeMapItem*);
@@ -306,7 +300,7 @@ public:
    * Used internally to enable tooltip.
    */
   void clearFreeRects();
-  Q3PtrList<QRect>* freeRects() const { return _freeRects; }
+  const QList<QRect>& freeRects() const { return _freeRects; }
   void addFreeRect(const QRect& r);
 
   /**
@@ -381,7 +375,7 @@ private:
 
   // temporary layout
   QRect _rect;
-  Q3PtrList<QRect>* _freeRects;
+  QList<QRect> _freeRects;
   int _depth;
 
   // temporary self value (when using level skipping)
@@ -407,6 +401,7 @@ public:
    */
   enum SelectionMode { Single, Multi, Extended, NoSelection };
 
+  /* The widget gets owner of the base item */
   explicit TreeMapWidget(TreeMapItem* base, QWidget* parent=0);
   ~TreeMapWidget();
 
@@ -466,7 +461,7 @@ public:
 
   /**
    * Selects or unselects items in a range.
-   * This is needed internally for Shift-Click in Extented mode.
+   * This is needed internally for Shift-Click in Extended mode.
    * Range means for a hierarchical widget:
    * - select/unselect i1 and i2 according selected
    * - search common parent of i1 and i2, and select/unselect the
@@ -510,6 +505,13 @@ public:
   void setShadingEnabled(bool s);
   bool isShadingEnabled() const { return _shading; }
 
+  /* Setting for a whole depth level: draw 3D frame (default) or solid */
+  void drawFrame(int d, bool b);
+  bool drawFrame(int d) const { return (d<4)?_drawFrame[d]:true; }
+
+  /* Setting for a whole depth level: draw items (default) or transparent */
+  void setTransparent(int d, bool b);
+  bool isTransparent(int d) const { return (d<4)?_transparent[d]:false; }
 
   /**
    * Items usually have a size proportional to their value().
@@ -684,10 +686,8 @@ protected:
   void mouseDoubleClickEvent( QMouseEvent * );
   void keyPressEvent( QKeyEvent* );
   void paintEvent( QPaintEvent * );
-  void resizeEvent( QResizeEvent * );
-  void showEvent( QShowEvent * );
   void fontChange( const QFont& );
-  bool event( QEvent* e );
+  bool event(QEvent *event);
 
 private:
   TreeMapItemList diff(TreeMapItemList&, TreeMapItemList&);
@@ -700,11 +700,11 @@ private:
   void drawItem(QPainter* p, TreeMapItem*);
   void drawItems(QPainter* p, TreeMapItem*);
   bool horizontal(TreeMapItem* i, const QRect& r);
-  void drawFill(TreeMapItem*,QPainter* p, QRect& r);
-  void drawFill(TreeMapItem*,QPainter* p, QRect& r,
-                TreeMapItemListIterator it, int len, bool goBack);
-  bool drawItemArray(QPainter* p, TreeMapItem*, QRect& r, double,
-                     TreeMapItemListIterator it, int len, bool);
+  void drawFill(TreeMapItem*,QPainter* p, const QRect& r);
+  void drawFill(TreeMapItem*,QPainter* p, const QRect& r,
+		TreeMapItemList* list, int idx, int len, bool goBack);
+  bool drawItemArray(QPainter* p, TreeMapItem*, const QRect& r, double,
+		     TreeMapItemList* list, int idx, int len, bool);
   bool resizeAttr(int);
 
   TreeMapItem* _base;
@@ -717,13 +717,14 @@ private:
     bool visible, forced;
     DrawParams::Position pos;
   };
-  Q3ValueVector<FieldAttr> _attr;
+  QVector<FieldAttr> _attr;
 
   SelectionMode _selectionMode;
   TreeMapItem::SplitMode _splitMode;
   int _visibleWidth, _stopArea, _minimalArea, _borderWidth;
   bool _reuseSpace, _skipIncorrectBorder, _drawSeparators, _shading;
   bool _allowRotation;
+  bool _transparent[4], _drawFrame[4];
   TreeMapItem * _needsRefresh;
   TreeMapItemList _selection;
   int _markNo;
