@@ -20,12 +20,20 @@
 
 #include "searchbar.h"
 
-#include "SearchSuggestion.h"
+#include "OpenSearchManager.h"
+#include "WebShortcutWidget.h"
 
 #include <unistd.h>
 #include <QLineEdit>
 #include <QApplication>
+#include <DOM/Element>
+#include <DOM/HTMLCollection>
+#include <DOM/HTMLDocument>
+#include <DOM/HTMLElement>
+#include <DOM/Node>
+#include <DOM/NodeList>
 #include <kaction.h>
+#include <KBuildSycocaProgressDialog>
 #include <KCompletionBox>
 #include <kconfig.h>
 #include <ksharedconfig.h>
@@ -50,8 +58,8 @@
 #include <qstyle.h>
 #include <QPixmap>
 #include <QMouseEvent>
-#include <QtGui/QCompleter>
 #include <QtCore/QTimer>
+#include <QtDBus/QtDBus>
 
 K_PLUGIN_FACTORY(SearchBarPluginFactory, registerPlugin<SearchBarPlugin>();)
 K_EXPORT_PLUGIN(SearchBarPluginFactory("searchbarplugin"))
@@ -63,7 +71,7 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
     m_searchMode(UseSearchProvider),
     m_urlEnterLock(false),
     m_process(0),
-    m_searchSuggestion(new SearchSuggestion(this))
+    m_openSearchManager(new OpenSearchManager(this))
 {
     m_searchCombo = new SearchBarCombo(0);
     m_searchCombo->lineEdit()->installEventFilter(this);
@@ -75,6 +83,7 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
     connect(m_searchCombo, SIGNAL(suggestionEnabled(bool)), this, SLOT(enableSuggestion(bool)));
 
     m_popupMenu = 0;
+    m_addWSWidget = 0;
 
     m_searchComboAction = actionCollection()->addAction("toolbar_search_bar");
     m_searchComboAction->setText(i18n("Search Bar"));
@@ -90,9 +99,12 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
 
     // parent is the KonqMainWindow and we want to listen to PartActivateEvent events.
     parent->installEventFilter(this);
+
     connect(m_searchCombo->lineEdit(), SIGNAL(textEdited(const QString &)), SLOT(searchTextChanged(const QString &)));
-    connect(m_searchSuggestion, SIGNAL(suggestionReceived(const QStringList &)),
+    connect(m_openSearchManager, SIGNAL(suggestionReceived(const QStringList &)),
             SLOT(addSearchSuggestion(const QStringList &)));
+    connect(m_openSearchManager, SIGNAL(openSearchEngineAdded(const QString &, const QString &, const QString &)),
+            SLOT(openSearchEngineAdded(const QString &, const QString &, const QString &)));
     m_timer = new QTimer(this);
     m_timer->setSingleShot(true);
     connect(m_timer, SIGNAL(timeout()), SLOT(requestSuggestion()));
@@ -116,6 +128,8 @@ bool SearchBarPlugin::eventFilter(QObject *o, QEvent *e)
     if (qobject_cast<KMainWindow*>(o) && KParts::PartActivateEvent::test(e)) {
         KParts::PartActivateEvent* partEvent = static_cast<KParts::PartActivateEvent *>(e);
         m_part = qobject_cast<KParts::ReadOnlyPart *>(partEvent->part());
+        // Delay since when destroying tabs part 0 gets activated for a bit, before the proper part
+        QTimer::singleShot(0, this, SLOT(updateComboVisibility()));
     }
     else if (o == m_searchCombo->lineEdit() && e->type() == QEvent::KeyPress) {
         QKeyEvent *k = (QKeyEvent *)e;
@@ -233,24 +247,23 @@ void SearchBarPlugin::setIcon()
         KUriFilterData data;
         QStringList list;
         list << "kurisearchfilter" << "kuriikwsfilter";
-        if ( !m_currentEngine.isEmpty() )
+        if (!m_currentEngine.isEmpty())
             service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
         if (service) {
             const QString searchProviderPrefix = service->property("Keys").toStringList().first() + m_delimiter;
             data.setData(searchProviderPrefix + "some keyword");
         }
 
-        if (service && KUriFilter::self()->filterUri(data, list))
-        {
+        if (service && KUriFilter::self()->filterUri(data, list)) {
             QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
             if (iconPath.isEmpty()) {
                 m_searchIcon = SmallIcon("unknown");
-            } else {
+            }
+            else {
                 m_searchIcon = QPixmap(iconPath);
             }
         }
-        else
-        {
+        else {
             m_searchIcon = SmallIcon("google");
         }
     }
@@ -282,7 +295,7 @@ void SearchBarPlugin::showSelectionMenu()
         m_popupMenu->addAction(KIcon("edit-find"), i18n("Find in This Page"), this, SLOT(useFindInThisPage()));
         m_popupMenu->addSeparator();
 
-        int i=-1;
+        int i =- 1;
         for (QStringList::ConstIterator it = m_searchEngines.constBegin(); it != m_searchEngines.constEnd(); ++it) {
             i++;
             KService::Ptr service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(*it));
@@ -306,11 +319,35 @@ void SearchBarPlugin::showSelectionMenu()
             }
         }
 
+        m_popupMenu->addSeparator();
+
         m_popupMenu->addSeparator();
         m_popupMenu->addAction(KIcon("preferences-web-browser-shortcuts"), i18n("Select Search Engines..."),
                                this, SLOT(selectSearchEngines()));
-        connect(m_popupMenu, SIGNAL(triggered(QAction *)), SLOT(useSearchProvider(QAction *)));
+        connect(m_popupMenu, SIGNAL(triggered(QAction *)), SLOT(menuActionTriggered(QAction *)));
     }
+    else { // the popup menu is already deleted, don't try to delete each action in m_addSearchActions, otherwise kaboom
+        foreach (KAction *action, m_addSearchActions) {
+            m_popupMenu->removeAction(action);
+            delete action;
+        }
+        m_addSearchActions.clear();
+    }
+
+    QList<QAction *> actions = m_popupMenu->actions();
+    QAction *before = 0;
+    if (actions.size() > 1) {
+        before = actions[actions.size() - 2];
+    }
+
+    foreach (QString title, m_openSearchDescs.keys()) {
+        KAction *addSearchAction = new KAction(m_popupMenu);
+        addSearchAction->setText(i18n("Add ") + title + "...");
+        m_addSearchActions.append(addSearchAction);
+        addSearchAction->setData(QVariant::fromValue(title));
+        m_popupMenu->insertAction(before, addSearchAction);
+    }
+
     m_popupMenu->popup(m_searchCombo->mapToGlobal(QPoint(0, m_searchCombo->height() + 1)));
 }
 
@@ -320,18 +357,41 @@ void SearchBarPlugin::useFindInThisPage()
     setIcon();
 }
 
-void SearchBarPlugin::useSearchProvider(QAction *action)
+void SearchBarPlugin::menuActionTriggered(QAction *action)
 {
     bool ok = false;
     const int id = action->data().toInt(&ok);
-    if(!ok) {
-        // Not a search engine entry selected
+    if (ok) {
+        m_searchMode = UseSearchProvider;
+        m_currentEngine = m_searchEngines.at(id);
+        setIcon();
+        KService::Ptr service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
+        m_openSearchManager->setSearchProvider(m_currentEngine);
+        m_searchCombo->lineEdit()->selectAll();
         return;
     }
-    m_searchMode = UseSearchProvider;
-    m_currentEngine = m_searchEngines.at(id);
-    setIcon();
-    m_searchCombo->lineEdit()->selectAll();
+
+    QString openSearchTitle = action->data().toString();
+    if (!openSearchTitle.isEmpty()) {
+        KHTMLPart *part = dynamic_cast<KHTMLPart *>(m_part);
+        if (part) {
+            QString openSearchHref = m_openSearchDescs.value(openSearchTitle);
+            KUrl url;
+            if (QUrl(openSearchHref).isRelative()) {
+                KUrl docUrl = part->url();
+                QString host = docUrl.scheme() + "://" + docUrl.host();
+                if (docUrl.port() != -1) {
+                    host += ":" + QString::number(docUrl.port());
+                }
+                url = KUrl(docUrl, openSearchHref);
+            }
+            else {
+                url = KUrl(openSearchHref);
+            }
+            kDebug() << "Adding open search Engine: " << openSearchTitle << " : " << openSearchHref;
+            m_openSearchManager->addOpenSearchEngine(url, openSearchTitle);
+        }
+    }
 }
 
 void SearchBarPlugin::selectSearchEngines()
@@ -381,42 +441,31 @@ void SearchBarPlugin::configurationChanged()
     for (QStringList::ConstIterator it = favoriteEngines.constBegin(); it != favoriteEngines.constEnd(); ++it)
         if(*it!=engine)
             m_searchEngines << *it;
-
-    if(engine.isEmpty()) {
-        m_providerName = "Google";
-    } else {
-        KDesktopFile file("services", "searchproviders/" + engine + ".desktop");
-        m_providerName = file.readName();
-    }
+    m_addSearchActions.clear();
 
     config = KConfigGroup(KGlobal::config(), "SearchBar");
     m_searchMode = (SearchModes) config.readEntry("Mode", (int) UseSearchProvider);
     m_currentEngine = config.readEntry("CurrentEngine", engine);
 
-    if (m_currentEngine.isEmpty())
-        m_currentEngine = "google";
-
     m_suggestionEnabled = config.readEntry("SuggestionEnabled", true);
     m_searchCombo->setSuggestionEnabled(m_suggestionEnabled);
-    setIcon();
-}
 
-void SearchBarPlugin::partChanged(KParts::Part *newPart)
-{
-    m_part = qobject_cast<KParts::ReadOnlyPart*>(newPart);
-    //Delay since when destroying tabs part 0 gets activated for a bit, before the proper part
-    QTimer::singleShot(0, this, SLOT(updateComboVisibility()));
+    KService::Ptr service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
+
+    m_openSearchManager->setSearchProvider(m_currentEngine);
+    setIcon();
 }
 
 void SearchBarPlugin::updateComboVisibility()
 {
-    if (!m_part|| m_searchComboAction->associatedWidgets().isEmpty())
-    {
-        m_searchCombo->setPluginActive(false);
-        m_searchCombo->hide();
-    } else {
-        m_searchCombo->setPluginActive(true);
-        m_searchCombo->show();
+    bool isBrowser = m_part && dynamic_cast<KHTMLPart *>(m_part);
+
+    m_searchComboAction->setVisible(isBrowser && !m_searchComboAction->associatedWidgets().isEmpty());
+
+    m_openSearchDescs.clear();
+    if (isBrowser) {
+        connect(m_part, SIGNAL(completed()), this, SLOT(HTMLDocLoaded()));
+        connect(m_part, SIGNAL(started(KIO::Job *)), this, SLOT(HTMLLoadingStarted()));
     }
 }
 
@@ -430,7 +479,7 @@ void SearchBarPlugin::searchTextChanged(const QString &text)
     m_searchCombo->clearSuggestions();
 
     if (m_suggestionEnabled && m_searchMode != FindInThisPage &&
-            m_searchSuggestion->isEngineAvailable(m_currentEngine) && !text.isEmpty()) {
+            m_openSearchManager->isSuggestionAvailable() && !text.isEmpty()) {
         if (m_timer->isActive()) {
             m_timer->stop();
         }
@@ -442,7 +491,7 @@ void SearchBarPlugin::searchTextChanged(const QString &text)
 
 void SearchBarPlugin::requestSuggestion() {
     if (!m_searchCombo->lineEdit()->text().isEmpty()) {
-        m_searchSuggestion->requestSuggestion(m_currentEngine, m_searchCombo->lineEdit()->text());
+        m_openSearchManager->requestSuggestion(m_searchCombo->lineEdit()->text());
     }
 }
 
@@ -451,14 +500,102 @@ void SearchBarPlugin::enableSuggestion(bool enable)
     m_suggestionEnabled = enable;
 }
 
+void SearchBarPlugin::HTMLDocLoaded()
+{
+    KHTMLPart *khtmlPart = static_cast<KHTMLPart *>(m_part);
+
+    DOM::HTMLDocument htmlDoc = khtmlPart->htmlDocument();
+    DOM::NodeList headNL = htmlDoc.getElementsByTagName("head");
+    if (headNL.length() < 1) {
+        return;
+    }
+
+    DOM::Node headNode = headNL.item(0);
+    if (headNode.nodeType() != 1) {
+        return;
+    }
+
+    DOM::HTMLElement headEl = (DOM::HTMLElement) headNode;
+    DOM::NodeList linkNL = headEl.getElementsByTagName("link");
+
+    for (unsigned int i = 0; i < linkNL.length(); i++) {
+        if (linkNL.item(i).nodeType() == 1) {
+            DOM::HTMLElement linkEl = (DOM::HTMLElement)linkNL.item(i);
+            if (linkEl.getAttribute("rel") == "search" && linkEl.getAttribute("type") == "application/opensearchdescription+xml") {
+                if (headEl.getAttribute("profile") != "http://a9.com/-/spec/opensearch/1.1/") {
+                    kWarning() << "Warning: there is no profile attribute or wrong profile attribute in <head>, as specified by open search specification 1.1";
+                }
+
+                QString title = linkEl.getAttribute("title").string();
+                QString href = linkEl.getAttribute("href").string();
+                m_openSearchDescs.insert(title, href);
+            }
+        }
+    }
+}
+
+void SearchBarPlugin::openSearchEngineAdded(const QString &name, const QString &searchUrl, const QString &fileName)
+{
+    kDebug() << "New Open Search Engine Added: " << name << ", searchUrl " << searchUrl;
+    QString path = KGlobal::mainComponent().dirs()->saveLocation("services", "searchproviders/");
+    
+    KConfig _service(path + fileName + ".desktop", KConfig::SimpleConfig);
+    KConfigGroup service(&_service, "Desktop Entry");
+    service.writeEntry("Type", "Service");
+    service.writeEntry("ServiceTypes", "SearchProvider");
+    service.writeEntry("Name", name);
+    service.writeEntry("Query", searchUrl);
+    service.writeEntry("Keys", fileName);
+    // TODO
+    service.writeEntry("Charset", "" /* provider->charset() */);
+
+    // we might be overwriting a hidden entry
+    service.writeEntry("Hidden", false);
+
+    // Show the add web shortcut widget
+    if (!m_addWSWidget) {
+        m_addWSWidget = new WebShortcutWidget(m_searchCombo);
+        m_addWSWidget->setWindowFlags(Qt::Popup);
+
+        connect(m_addWSWidget, SIGNAL(webShortcutSet(const QString &, const QString &, const QString &)),
+                this, SLOT(webShortcutSet(const QString &, const QString &, const QString &)));
+    }
+
+    QPoint pos = m_searchCombo->mapToGlobal(QPoint(0, m_searchCombo->height() + 1));
+    m_addWSWidget->setGeometry(QRect(pos, m_addWSWidget->size()));
+    m_addWSWidget->show(name, fileName);
+}
+
+void SearchBarPlugin::webShortcutSet(const QString &name, const QString &webShortcut, const QString &fileName)
+{
+    kDebug() << "Web shortcut for: " << name << "set to: " << webShortcut;
+    QString path = KGlobal::mainComponent().dirs()->saveLocation("services", "searchproviders/");
+    KConfig _service(path + fileName + ".desktop", KConfig::SimpleConfig);
+    KConfigGroup service(&_service, "Desktop Entry");
+    service.writeEntry("Keys", webShortcut);
+
+    // Update filters in running applications...
+    QDBusMessage msg = QDBusMessage::createSignal("/", "org.kde.KUriFilterPlugin", "configure");
+    QDBusConnection::sessionBus().send(msg);
+
+    // If the providers changed, tell sycoca to rebuild its database...
+    KBuildSycocaProgressDialog::rebuildKSycoca(m_searchCombo);
+}
+
+void SearchBarPlugin::HTMLLoadingStarted()
+{
+    // reset the open search availability, so that if there is previously detected engine,
+    // it will not be shown
+    m_openSearchDescs.clear();
+}
+
 void SearchBarPlugin::addSearchSuggestion(const QStringList &suggestions)
 {
     m_searchCombo->setSuggestionItems(suggestions);
 }
 
 SearchBarCombo::SearchBarCombo(QWidget *parent) :
-    KHistoryComboBox(true, parent),
-    m_pluginActive(true)
+    KHistoryComboBox(true, parent)
 {
     setDuplicatesEnabled(false);
     setFixedWidth(180);
@@ -535,11 +672,6 @@ void SearchBarCombo::historyCleared()
     setIcon(m_icon);
 }
 
-void SearchBarCombo::setPluginActive(bool pluginActive)
-{
-    m_pluginActive = pluginActive;
-}
-
 void SearchBarCombo::setSuggestionItems(const QStringList &suggestions)
 {
     if (!m_suggestions.isEmpty()) {
@@ -571,12 +703,6 @@ void SearchBarCombo::clearSuggestions()
     m_suggestions.clear();
 }
 
-void SearchBarCombo::show()
-{
-    if (m_pluginActive) {
-        KHistoryComboBox::show();
-    }
-}
 
 void SearchBarCombo::addEnableMenuItem(QMenu *menu)
 {
